#include <stdio.h>
#include <stdlib.h>
#include <wand/magick-wand.h>

#include "Viewer.h"
#include <jni.h>

/*
 * Class:     Viewer
 * Method:    getImage
 * Signature: ()Ljava/awt/image/BufferedImage;
 */
JNIEXPORT jobject JNICALL Java_Viewer_getImage
  (JNIEnv *env, jclass klass, jobject buffer) {
	jintArray javaPixels;
	jint *pixels;
	int width, height;
	jclass bi_class;
	jmethodID const_mid, setrgb_mid;
	jfieldID type_fid;
	jint type;
	jobject bufferedimage;

	// Creates context object for loading an image.
	MagickWand *magick_wand = NewMagickWand();

	// Reads image data from file.
	if(!MagickReadImageBlob(
		magick_wand,
		(*env)->GetDirectBufferAddress(env, buffer),
		(*env)->GetDirectBufferCapacity(env, buffer)
		)) {

		magick_wand = DestroyMagickWand(magick_wand);

		return NULL;
	}

	// Retrieves image dimension in order to create a properly sized buffer array.
	width = MagickGetImageWidth(magick_wand);
	height = MagickGetImageHeight(magick_wand);

	// Creates an int array for holding the pixel data
	// Note: sizeof(jint) == 4 is important here
	javaPixels = (*env)->NewIntArray(env, width*height);

	// Retrieves the jint array (remember that this MAY be a copy of
	// the internal data)
	pixels = (*env)->GetIntArrayElements(env, javaPixels, 0);

	// Copies the pixel data into the java array
	if(!MagickGetImagePixels(magick_wand, 0, 0, width, height, "BGRA", CharPixel, (void *) pixels)) {
		magick_wand = DestroyMagickWand(magick_wand);

		return NULL;
	}

	// Releases the lock or copies the new data back into the Java array object.
	(*env)->ReleaseIntArrayElements(env, javaPixels, pixels, 0);

	magick_wand = DestroyMagickWand(magick_wand);

	// Prepares the instantiation of a BufferedImage.
	bi_class = (*env)->FindClass(env, "java/awt/image/BufferedImage");
	const_mid = (*env)->GetMethodID(env, bi_class, "<init>", "(III)V");

	// Prepares accessing the BufferedImage.TYPE_INT_ARGB value for the constructor
	// call.
	type_fid = (*env)->GetStaticFieldID(env, bi_class, "TYPE_INT_ARGB", "I");
	type = (*env)->GetStaticIntField(env, bi_class, type_fid);

	// Creates a BufferedImage instance with ARGB data.
	bufferedimage = (*env)->NewObject(env, bi_class, const_mid, width, height, type);

	// Copies the pixel data into the image making it available for the Java world.
	setrgb_mid = (*env)->GetMethodID(env, bi_class, "setRGB", "(IIII[III)V");
	(*env)->CallVoidMethod(env, bufferedimage, setrgb_mid, 0, 0, width, height, javaPixels, 0, width);

	return bufferedimage;
}
